use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'bin/tidyall',
    'lib/Code/TidyAll.pm',
    'lib/Code/TidyAll/Cache.pm',
    'lib/Code/TidyAll/CacheModel.pm',
    'lib/Code/TidyAll/CacheModel/Shared.pm',
    'lib/Code/TidyAll/Config/INI/Reader.pm',
    'lib/Code/TidyAll/Git/Precommit.pm',
    'lib/Code/TidyAll/Git/Prereceive.pm',
    'lib/Code/TidyAll/Git/Util.pm',
    'lib/Code/TidyAll/Plugin.pm',
    'lib/Code/TidyAll/Plugin/CSSUnminifier.pm',
    'lib/Code/TidyAll/Plugin/GenericTransformer.pm',
    'lib/Code/TidyAll/Plugin/GenericValidator.pm',
    'lib/Code/TidyAll/Plugin/JSBeautify.pm',
    'lib/Code/TidyAll/Plugin/JSHint.pm',
    'lib/Code/TidyAll/Plugin/JSLint.pm',
    'lib/Code/TidyAll/Plugin/JSON.pm',
    'lib/Code/TidyAll/Plugin/MasonTidy.pm',
    'lib/Code/TidyAll/Plugin/PHPCodeSniffer.pm',
    'lib/Code/TidyAll/Plugin/PerlCritic.pm',
    'lib/Code/TidyAll/Plugin/PerlTidy.pm',
    'lib/Code/TidyAll/Plugin/PerlTidySweet.pm',
    'lib/Code/TidyAll/Plugin/PodChecker.pm',
    'lib/Code/TidyAll/Plugin/PodSpell.pm',
    'lib/Code/TidyAll/Plugin/PodTidy.pm',
    'lib/Code/TidyAll/Plugin/SortLines.pm',
    'lib/Code/TidyAll/Result.pm',
    'lib/Code/TidyAll/Role/GenericExecutable.pm',
    'lib/Code/TidyAll/Role/HasIgnore.pm',
    'lib/Code/TidyAll/Role/RunsCommand.pm',
    'lib/Code/TidyAll/Role/Tempdir.pm',
    'lib/Code/TidyAll/SVN/Precommit.pm',
    'lib/Code/TidyAll/SVN/Util.pm',
    'lib/Code/TidyAll/Util.pm',
    'lib/Code/TidyAll/Util/Zglob.pm',
    'lib/Code/TidyAll/Zglob.pm',
    'lib/Test/Code/TidyAll.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/Basic.t',
    't/Conf.t',
    't/Git.t',
    't/Plugin-CSSUnminifier.t',
    't/Plugin-GenericTransformer.t',
    't/Plugin-GenericValidator.t',
    't/Plugin-JSBeautify.t',
    't/Plugin-JSHint.t',
    't/Plugin-JSLint.t',
    't/Plugin-MasonTidy.t',
    't/Plugin-PHPCodeSniffer.t',
    't/Plugin-PerlCritic.t',
    't/Plugin-PerlTidy.t',
    't/Plugin-PerlTidySweet.t',
    't/Plugin-PodChecker.t',
    't/Plugin-PodSpell.t',
    't/Plugin-PodTidy.t',
    't/Plugin-SortLines.t',
    't/Util.t',
    't/Zglob.t',
    't/diff-on-tidy-error.t',
    't/helper-bin/exit.pl',
    't/helper-bin/generic-transformer.pl',
    't/helper-bin/generic-validator.pl',
    't/helper-bin/raw-transformer.pl',
    't/inc1/Foo.pm',
    't/inc2/Bar.pm',
    't/lib/TestFor/Code/TidyAll/Basic.pm',
    't/lib/TestFor/Code/TidyAll/Conf.pm',
    't/lib/TestFor/Code/TidyAll/DiffOnTidyError.pm',
    't/lib/TestFor/Code/TidyAll/Git.pm',
    't/lib/TestFor/Code/TidyAll/Parallel.pm',
    't/lib/TestFor/Code/TidyAll/Plugin.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/CSSUnminifier.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/GenericTransformer.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/GenericValidator.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/JSBeautify.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/JSHint.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/JSLint.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/MasonTidy.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PHPCodeSniffer.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PerlCritic.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PerlTidy.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PerlTidySweet.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PodChecker.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PodSpell.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/PodTidy.pm',
    't/lib/TestFor/Code/TidyAll/Plugin/SortLines.pm',
    't/lib/TestFor/Code/TidyAll/SVN.pm',
    't/lib/TestFor/Code/TidyAll/SpacesInPaths.pm',
    't/lib/TestFor/Code/TidyAll/Util.pm',
    't/lib/TestFor/Code/TidyAll/Util/Zglob.pm',
    't/lib/TestHelper/Plugin/AToZ.pm',
    't/lib/TestHelper/Plugin/AlwaysPhonetic.pm',
    't/lib/TestHelper/Plugin/CheckUpper.pm',
    't/lib/TestHelper/Plugin/RepeatFoo.pm',
    't/lib/TestHelper/Plugin/ReverseFoo.pm',
    't/lib/TestHelper/Plugin/UpperText.pm',
    't/lib/TestHelper/Test/Class.pm',
    't/parallel.t',
    't/spaces-in-paths.t'
);

notabs_ok($_) foreach @files;
done_testing;
