use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
Adam
Alders
Andreas
Andy
Anwar
Bernhard
CPAN
CSSUnminifier
Cache
CacheModel
Code
Config
Crotty
Custodio
DROLSKY
DROLSKY's
Dave
Dostál
Finn
Fish
Fowler
GenericExecutable
GenericTransformer
GenericValidator
George
Git
Graham
Gregory
Grimes
Gruner
Hartzell
HasIgnore
Herzog
INI
JSBeautify
JSHint
JSLint
JSON
Jack
JavaScript
Joe
Jonathan
Kenneth
Knop
Mark
Martin
MasonTidy
Matsuno
Melo
Misc
Mohammad
Nick
Olaf
Oschwald
PHP
PHPCodeSniffer
PREREQS
Paulo
Pavel
PayPal
Pedro
PerlCritic
PerlTidy
PerlTidySweet
Plugin
PodChecker
PodSpell
PodTidy
Precommit
Prereceive
Raggett
Reader
Result
Ricardo
Role
Rolsky
Rolsky's
Romanov
RunsCommand
STDERR
STDIN
STDOUT
SVN
Schmalhofer
Sergey
Shared
Shlomi
Signes
Simakov
Smith
SortLines
Swartz
TIDYALL
Tempdir
Test
Thalhammer
TidyAll
Tokuhiro
Tonkin
Util
Vim
Vögele
Zglob
adam
andreas
andyjack
argv
ascii
atomicity
autarch
bin
borged
cmd
conf
config
cssunminifier
ctrl
dir
drolsky
filename
finn
georgewh
goschwald
gregoa
gregor
haarg
herrmann
iff
ispell
joe
js
jsbeautify
jshint
jslint
kenneth
lib
listref
lukasmai
mark
martin
masontidy
mauke
melo
mgrimes
mohammad
noll
npm
ntonkin
ok
olaf
ops
params
pathname
pauloscustodio
pdostal
perlcritic
perlcriticrc
perltidy
phpcs
plugins
podspell
podtidy
pre
prereqs
repo
repos
rjbs
ro
runtime
rw
shebang
shlomif
sourceforge
sromanov
svn
swartz
tidiers
tidyall
tidyall'd
tim
timestamped
timgimyee
ttl
txn
validator
validators
zglob
Ölwing
